package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.stream.Stream;

public final class InnerClasses extends Attribute implements Iterable<InnerClass> {
   private static final InnerClass[] EMPTY_INNER_CLASSE_ARRAY = new InnerClass[0];
   private InnerClass[] innerClasses;

   public InnerClasses(InnerClasses c) {
      this(c.getNameIndex(), c.getLength(), c.getInnerClasses(), c.getConstantPool());
   }

   InnerClasses(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
      this(nameIndex, length, (InnerClass[])null, constantPool);
      int classCount = input.readUnsignedShort();
      this.innerClasses = new InnerClass[classCount];

      for(int i = 0; i < classCount; ++i) {
         this.innerClasses[i] = new InnerClass(input);
      }

   }

   public InnerClasses(int nameIndex, int length, InnerClass[] innerClasses, ConstantPool constantPool) {
      super((byte)6, nameIndex, length, constantPool);
      this.innerClasses = innerClasses != null ? innerClasses : EMPTY_INNER_CLASSE_ARRAY;
   }

   public void accept(Visitor v) {
      v.visitInnerClasses(this);
   }

   public Attribute copy(ConstantPool constantPool) {
      InnerClasses c = (InnerClasses)this.clone();
      c.innerClasses = new InnerClass[this.innerClasses.length];
      Arrays.setAll(c.innerClasses, (i) -> {
         return this.innerClasses[i].copy();
      });
      c.setConstantPool(constantPool);
      return c;
   }

   public void dump(DataOutputStream file) throws IOException {
      super.dump(file);
      file.writeShort(this.innerClasses.length);
      InnerClass[] var2 = this.innerClasses;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         InnerClass innerClass = var2[var4];
         innerClass.dump(file);
      }

   }

   public InnerClass[] getInnerClasses() {
      return this.innerClasses;
   }

   public Iterator<InnerClass> iterator() {
      return Stream.of(this.innerClasses).iterator();
   }

   public void setInnerClasses(InnerClass[] innerClasses) {
      this.innerClasses = innerClasses != null ? innerClasses : EMPTY_INNER_CLASSE_ARRAY;
   }

   public String toString() {
      StringBuilder buf = new StringBuilder();
      buf.append("InnerClasses(");
      buf.append(this.innerClasses.length);
      buf.append("):\n");
      InnerClass[] var2 = this.innerClasses;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         InnerClass innerClass = var2[var4];
         buf.append(innerClass.toString(super.getConstantPool())).append("\n");
      }

      return buf.substring(0, buf.length() - 1);
   }
}
